#ifndef CoCoA_SparsePolyOps_Cyclotomicity_H
#define CoCoA_SparsePolyOps_Cyclotomicity_H

//   Copyright (c)  2022  John Abbott,  Anna M. Bigatti
//   Original author: Nico Mexis (2022)

//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BigInt.H"
#include "CoCoA/ring.H"

#include <vector>
// using std::vector;

namespace CoCoA
{

    /**
     * Provides a simple-to-compute bound for inverse
     * the Euler Totient function. All numbers k with
     * phi(k) = n will satisfy k < InversePhiBound(n).
     * See https://oeis.org/A355667
     */
    BigInt InversePhiBound(const unsigned long n);

    /**
     * Ported directly from C5 code without optimizations.
     * Used for testing current implementation of IsCyclotomic
     * and IsCyclotomic2 functions.
     */
    RingElem gcdList(const std::vector<RingElem> &L);

    /**
     * Ported directly from C5 code without optimizations.
     * Used for testing current implementation of IsCyclotomic
     * and IsCyclotomic2 functions. Checks are missing!
     */
    RingElem cyclotomic(const ring &P, const long K);

    /**
     * Graeffe approach
     */
    //unsigned long IsCyclotomicGraeffe(ConstRefRingElem f);

    /**
     * Inverse Phi method
     */
    //unsigned long IsCyclotomicIter(ConstRefRingElem f);

    /**
     * Chooses the right approach for the given polynomial
     */
    unsigned long IsCyclotomic(ConstRefRingElem f);

} // end of namespace CoCoA

// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/SparsePolyOps-Cyclotomicity.H,v 1.3 2022/11/30 15:17:43 abbott Exp $
// $Log: SparsePolyOps-Cyclotomicity.H,v $
// Revision 1.3  2022/11/30 15:17:43  abbott
// Summary: Removed IsCyclotomicGraeffe & IsCyclotomicIter
//
// Revision 1.2  2022/11/24 21:11:29  abbott
// Summary: Added copyright notice
//


#endif
