#ifndef CoCoA_SparsePolyOps_LRSDegeneracy_H
#define CoCoA_SparsePolyOps_LRSDegeneracy_H

//   Copyright (c)  2022  John Abbott,  Anna M. Bigatti
//   Original author: Nico Mexis (2022)

//   This file is part of the source of CoCoALib, the CoCoA Library.
//
//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/ring.H"

namespace CoCoA
{

  /**
   * Algorithm 1 (naive factor-approach)
   * NOT SUPPORTED AND NOT UPDATED
   */
  //unsigned long IsLRSDegenerateNaive(RingElem f);

  /**
   * Algorithm 2 (Graeffe iteration), used for reducible polynomials
   * NOT SUPPORTED AND NOT UPDATED
   */
  //unsigned long IsLRSDegenerateIter(RingElem f);

  /**
   * Checks for LRS-degeneracy of the given order
   */
  bool IsLRSDegenerateOrder(RingElem f, const unsigned long n);

  /**
   * Modular approach
   * Caller's responsibility to ensure that f is irreducible!
   * TODO: IrredFlag not needed anymore?!
   */
  unsigned long IsLRSDegenerateMod(RingElem f);

  /**
   * Chooses the right approach for the given polynomial
   */
  unsigned long IsLRSDegenerate(ConstRefRingElem f);

} // end of namespace CoCoA

// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/SparsePolyOps-LRSDegeneracy.H,v 1.3 2022/11/30 15:19:43 abbott Exp $
// $Log: SparsePolyOps-LRSDegeneracy.H,v $
// Revision 1.3  2022/11/30 15:19:43  abbott
// Summary: Removed unused flag arg
//
// Revision 1.2  2022/11/24 21:11:29  abbott
// Summary: Added copyright notice
//


#endif
