
#include "CoCoA/assert.H"
#include "CoCoA/BuildInfo.H"
#include "CoCoA/error.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/RingZZ.H"
#include "CoCoA/SparsePolyRing.H"
#include "CoCoA/SparsePolyOps-Cyclotomicity.H"

#include <iostream>
using std::cerr;
using std::endl;

namespace CoCoA
{

  void TestIsCyclotomic()
  {
    SparsePolyRing P = NewPolyRing(RingZZ(), symbols("x")); // ZZ[x];

    RingElem f = cyclotomic(P, 8);
    CoCoA_ASSERT_ALWAYS(IsCyclotomic(f) == 8);

    RingElem g = RingElem(P, "x^8-x^6+x^4-x^2+1");
    CoCoA_ASSERT_ALWAYS(IsCyclotomic(g) == 20);

    RingElem h = RingElem(P, "x^19+x^10+1");
    CoCoA_ASSERT_ALWAYS(IsCyclotomic(h) == 0);

    RingElem l = RingElem(P, "x^10+x^9-x^7-x^6-x^5-x^4-x^3+x+1");
    CoCoA_ASSERT_ALWAYS(IsCyclotomic(l) == 0);

    SparsePolyRing Pxy = NewPolyRing(RingZZ(), symbols("x,y")); // ZZ[x,y];
    RingElem m = RingElem(Pxy, "y^10+x^9-x^7-x^6-x^5-x^4-x^3+x+1");
    try
    {
      IsCyclotomic(m);
      CoCoA_ASSERT_ALWAYS(!"Non-univariate polynomials should throw in IsCyclotomic!");
    }
    catch (const CoCoA::ErrorInfo &err)
    {
      if (err != ERR::NotUnivariate) throw;
    }
  }

  void program()
  {
    GlobalManager CoCoAFoundations;

    TestIsCyclotomic();
  }

} // end of namespace CoCoA

//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    CoCoA::program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo &err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA Error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception &exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch (...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }

  CoCoA::BuildInfo::PrintAll(cerr);
  return 1;
}
